import numpy as np
from math import sqrt


def seprop(x):
    """
    Calculate the standard error for a proportion

    Usage:
    seprop(x)

    Arguments:
    x	Numeric variable with only values 0 and 1

    Examples:
    seprop([0, 1, 1, 1, 0, 0, 0])
    """
    p = np.nanmean(x)
    varprop = p * (1 - p)
    return sqrt(varprop / len(x))



def add_description(df, md="", path=""):
    """
    Add a description to a pandas data frame

    Arguments:
    df      A pandas data frame
    md      Data description in markdown format
    path    Path to a text file with the data description in markdown format

    Return:
    Pandas data frame with added description
    """
    if path != "":
        f = open(path, "r")
        md = f.read()
        f.close()
    elif md == "":
        print("Provide either text (markdown) or the path to a file")
        print("with the data description")

    df._metadata.append("description")
    df.description = md
    return df


def describe(df):
    """Print out data frame description attribute if available"""
    if hasattr(df, "description"):
        print(df.description)
    else:
        print("No description attribute available")
        return df.describe()


def ifelse(cond, if_true, if_false):
    """Oneline if-else like R"""
    if cond:
        return if_true
    else:
        return if_false



import numpy as np
import seaborn as sns


def prop_calc(df, group, rvar, lev):
    """
    Calculate proportions by a grouping variable

    Arguments:
    df          A data frame
    group       A variable in the data frame to group by
    rvar        The response variable to calculate proportions for
    lev         The level considered 'success'

    Return:
    A data frame with the grouping variable and the proportion of level
        "lev" in "rvar" for each value of the grouping variable
    """
    df["rvar_int"] = np.where(df[rvar] == lev, 1, 0)
    df = df.groupby(group)
    return (
        df.agg(prop=("rvar_int", "mean")).reset_index().rename(columns={"prop": rvar})
    )


def prop_plot(df, group, rvar, lev, breakeven=None):
    """
    Plot proportions by a grouping variable

    Arguments:
    df          A data frame
    group       A variable in the data frame to group by
    rvar        The response variable to calculate proportions for
    lev         The level considered 'success'
    breakeven   If numeric a horizontal line will be added at the
                   specified breakeven point
    """
    dfp = prop_calc(df, group, rvar, lev)
    cn = dfp.columns
    fig = sns.barplot(x=cn[0], y=cn[1], color="blue", data=dfp)
    fig.set(ylabel=f"Proportion of {cn[1]} = '{lev}'")
    if breakeven is not None:
        fig.axhline(breakeven, linestyle="dashed", linewidth=0.5)
    return fig



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
from statsmodels.stats.outliers_influence import variance_inflation_factor


def or_conf_int(fitted, alpha=0.05, intercept=False, dec=3):
    """
    Confidence interval for Odds ratios

    Arguments:
    fitted   A fitted logistic regression model
    alpha   Significance level
    dec     Number of decimal places

    Return:
    A dataframe with the Odd-ratios and confidence interval
    """
    df = pd.DataFrame(np.exp(fitted.params), columns=["OR"])
    low, high = [100 * alpha / 2, 100 * (1 - (alpha / 2))]
    df[[f"{low}%", f"{high}%"]] = np.exp(fitted.conf_int(alpha=alpha))

    if dec is not None:
        df = df.round(dec)

    df = df.reset_index()

    if intercept is False:
        df = df.loc[df["index"] != "Intercept"]

    return df


def or_plot(fitted, alpha=0.05, intercept=False):
    """
    Odds ratio plot

    Arguments:
    fitted       A fitted logistic regression fitted
    alpha       Significance level
    intercept   Include intercept in plot True or False
    """
    df = or_conf_int(fitted, alpha=alpha, intercept=intercept, dec=None).iloc[::-1]

    low, high = [100 * alpha / 2, 100 * (1 - (alpha / 2))]
    err = [df["OR"] - df[f"{low}%"], df[f"{high}%"] - df["OR"]]

    fig, ax = plt.subplots()
    ax.axvline(1, ls="dashdot")
    ax.errorbar(x="OR", y="index", data=df, xerr=err, fmt="none")
    ax.scatter(x="OR", y="index", data=df)
    ax.set_xscale("log")
    ax.xaxis.set_minor_formatter(ticker.NullFormatter())
    ax.xaxis.set_major_locator(ticker.LogLocator(subs=[0.1, 0.2, 0.5, 1, 2, 5, 10]))
    ax.xaxis.set_major_formatter(ticker.StrMethodFormatter("{x:.1f}"))
    ax.set(xlabel="Odds-ratio")
    return ax


def vif(model, dec=3):
    """
    Calculate the Variance Inflation Factor (VIF) associated with each
    exogenous variable

    WIP port the VIF calculation from R's car:::vif.default to Python

    Arguments:
    model   A specified model that has not yet been fit
    dec     Decimal places to use in rounding

    Return:
    A dataframe sorted by VIF score
    """
    vif = [variance_inflation_factor(model.exog, i) for i in range(model.exog.shape[1])]
    df = pd.DataFrame(model.exog_names, columns=["variable"])
    df["vif"] = vif
    df["Rsq"] = 1 - 1 / df["vif"]

    if "Intercept" in model.exog_names:
        df = df.loc[df["variable"] != "Intercept"]

    df = df.sort_values("vif", ascending=False)
    df.index = range(df.shape[0])

    if dec is not None:
        df = df.round(dec)

    return df
